Method: Flydata::Source::ParseDumpAndSend#value_converters

Defined in:
lib/flydata/source/parse_dump_and_send.rb

#value_convertersObject

Public Interface: Value Converters

Returns a hash of value converters which convert source values to FlyData values. Hash key is a data type for which the converter is called. A conveter takes a source value and returns a converted value.



18
19
20
# File 'lib/flydata/source/parse_dump_and_send.rb', line 18

def value_converters
  raise UnsupportedSourceError,  "subclass must implement"
end