Class: Flydata::MysqlCompatibilityCheck
Defined Under Namespace
Classes: MysqlCompatibilityError
Constant Summary
collapse
- SELECT_QUERY_TMPLT =
"SELECT %s"
- BINLOG_RETENTION_HOURS =
24
- SELECT_TABLE_INFO_TMPLT =
"SELECT table_name, table_type, engine FROM information_schema.tables WHERE table_schema = '%s' and table_name in(%s)"
Instance Method Summary
collapse
#check
#before_logging, #log_error_stderr, #log_info_stdout, #log_warn_stderr
Constructor Details
def initialize(de_hash, dump_dir=nil)
84
85
86
87
88
89
90
91
|
# File 'lib/flydata/compatibility_check.rb', line 84
def initialize(dp_hash, de_hash, options={})
super
@db_opts = [:host, :port, :username, :password, :database, :ssl_ca].inject({}) {|h, sym| h[sym] = de_hash[sym.to_s]; h}
@db_opts[:sslca] = @db_opts[:ssl_ca] @dump_dir = options[:dump_dir] || nil
@backup_dir = options[:backup_dir] || nil
@tables = de_hash['tables']
end
|
Instance Method Details
#check_mysql_binlog_retention ⇒ Object
179
180
181
182
183
184
185
186
187
188
189
190
|
# File 'lib/flydata/compatibility_check.rb', line 179
def check_mysql_binlog_retention
client = Mysql2::Client.new(@db_opts)
begin
if is_rds?(@db_opts[:host])
run_rds_retention_check(client)
else
run_mysql_retention_check(client)
end
ensure
client.close
end
end
|
#check_mysql_parameters_compat ⇒ Object
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
|
# File 'lib/flydata/compatibility_check.rb', line 143
def check_mysql_parameters_compat
sys_var_to_check = {'@@binlog_format'=>'ROW', '@@binlog_checksum'=>'NONE', '@@log_bin_use_v1_row_events'=>1, '@@log_slave_updates'=>1}
errors={}
client = Mysql2::Client.new(@db_opts)
begin
sys_var_to_check.each_key do |sys_var|
sel_query = SELECT_QUERY_TMPLT % sys_var
begin
result = client.query(sel_query)
unless result.first[sys_var] == sys_var_to_check[sys_var]
errors[sys_var]=result.first[sys_var]
end
rescue Mysql2::Error => e
if e.message =~ /Unknown system variable/
unless e.message =~ /(binlog_checksum|log_bin_use_v1_row_events)/
errors[sys_var] = false
end
else
raise e
end
end
end
ensure
client.close
end
unless errors.empty?
error_explanation = ""
errors.each_key do |err_key|
error_explanation << "\n * #{err_key} is #{errors[err_key]} but should be #{sys_var_to_check[err_key]}"
end
raise MysqlCompatibilityError, "These system variable(s) are not the correct value: #{error_explanation}\n Please change these system variables for FlyData Sync to run correctly"
end
end
|
#check_mysql_protocol_tcp_compat ⇒ Object
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
|
# File 'lib/flydata/compatibility_check.rb', line 125
def check_mysql_protocol_tcp_compat
query = Util::MysqlUtil.generate_mysql_show_grants_cmd(@db_opts)
Open3.popen3(query) do |stdin, stdout, stderr|
stdin.close
while !stderr.eof?
lines = []
while line = stderr.gets; lines << line.strip; end
err_reason = lines.join(" ")
log_error("Error occured during access to mysql server.", {err: err_reason})
unless /Warning: Using a password on the command line interface can be insecure/ === err_reason
raise MysqlCompatibilityError, "Cannot connect to MySQL database. Please make sure you can connect with this command:\n $ mysql -u #{@db_opts[:username]} -h #{@db_opts[:host]} -P #{@db_opts[:port]} #{@db_opts[:database]} --protocol=tcp -p"
end
end
end
end
|
#check_mysql_table_types ⇒ Object
If table_type='VIEW' or engine='MEMORY', raise error.
209
210
211
212
213
214
215
216
217
218
219
220
221
222
|
# File 'lib/flydata/compatibility_check.rb', line 209
def check_mysql_table_types
return if @tables.empty?
client = Mysql2::Client.new(@db_opts)
sel_query = SELECT_TABLE_INFO_TMPLT % [client.escape(@db_opts[:database]), @tables.collect{|t| "'#{client.escape(t)}'"}.join(", ")]
begin
invalid_tables = []
client.query(sel_query).each do |r|
invalid_tables.push(r['table_name']) if r['table_type'] == 'VIEW' || r['engine'] == 'MEMORY'
end
raise MysqlCompatibilityError, "FlyData does not support VIEW and MEMORY ENGINE table. Remove following tables from data entry: #{invalid_tables.join(", ")}" unless invalid_tables.empty?
ensure
client.close
end
end
|
#check_mysql_user_compat ⇒ Object
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
|
# File 'lib/flydata/compatibility_check.rb', line 102
def check_mysql_user_compat
client = Mysql2::Client.new(@db_opts)
grants_sql = "SHOW GRANTS"
correct_db = ["ON (\\*|#{@db_opts[:database]})","TO '#{@db_opts[:username]}"]
necessary_permission_fields= ["SELECT","RELOAD","LOCK TABLES","REPLICATION SLAVE","REPLICATION CLIENT"]
all_privileges_field= ["ALL PRIVILEGES"]
result = client.query(grants_sql)
client.close
missing_priv = []
result.each do |res|
res_value = res.values.first
if correct_db.all? {|perm| res_value.match(perm)}
necessary_permission_fields.each do |priv|
missing_priv << priv unless res_value.match(priv)
end
return true if missing_priv.empty? or all_privileges_field.all? {|d| res_value.match(d)}
end
end
raise MysqlCompatibilityError, "The user '#{@db_opts[:username]}' does not have the correct permissions to run FlyData Sync\n * These privileges are missing: #{missing_priv.join(", ")}"
end
|
#check_writing_permissions ⇒ Object
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
|
# File 'lib/flydata/compatibility_check.rb', line 192
def check_writing_permissions
write_errors = []
paths_to_check = ["~/.flydata"]
paths_to_check << @dump_dir unless @dump_dir.to_s.empty?
paths_to_check << @backup_dir unless @backup_dir.to_s.empty?
paths_to_check.each do |path|
full_path = File.expand_path(path)
full_path = File.dirname(full_path) unless File.directory?(full_path)
write_errors << full_path unless File.writable?(full_path) and File.executable?(full_path)
end
unless write_errors.empty?
error_dir = write_errors.join(", ")
raise MysqlCompatibilityError, "We cannot access the directories: #{error_dir}"
end
end
|
#is_rds?(hostname) ⇒ Boolean
262
263
264
|
# File 'lib/flydata/compatibility_check.rb', line 262
def is_rds?(hostname)
hostname.match(/rds.amazonaws.com$/) != nil
end
|
#print_errors ⇒ Object
93
94
95
96
97
98
99
100
|
# File 'lib/flydata/compatibility_check.rb', line 93
def print_errors
return if @errors.empty?
log_error_stderr "There may be some compatibility issues with your MySQL credentials: "
@errors.each do |error|
log_error_stderr " * #{error.message}"
end
raise "Please correct these errors if you wish to run FlyData Sync"
end
|
#run_mysql_retention_check(mysql_client) ⇒ Object
224
225
226
227
228
229
230
231
232
|
# File 'lib/flydata/compatibility_check.rb', line 224
def run_mysql_retention_check(mysql_client)
expire_logs_days_limit = BINLOG_RETENTION_HOURS / 24
sel_query = SELECT_QUERY_TMPLT % '@@expire_logs_days'
result = mysql_client.query(sel_query)
if result.first["@@expire_logs_days"]!=0 and result.first["@@expire_logs_days"] <= expire_logs_days_limit
raise MysqlCompatibilityError, "Binary log retention is too short\n " +
" We recommend the system variable '@@expire_logs_days' to be either set to 0 or at least #{expire_logs_days_limit} days"
end
end
|
#run_rds_retention_check(mysql_client) ⇒ Object
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
|
# File 'lib/flydata/compatibility_check.rb', line 234
def run_rds_retention_check(mysql_client)
sql_query = "call mysql.rds_show_configuration;"
begin
result = mysql_client.query(sql_query)
if result.first["name"]=="binlog retention hours"
if result.first["value"].nil? or result.first["value"].to_i <= BINLOG_RETENTION_HOURS
raise MysqlCompatibilityError, "Binary log retention is too short\n" +
" We recommend setting RDS binlog retention to be at least #{BINLOG_RETENTION_HOURS} hours. To do this, run this on your RDS MySQL database:\n" +
" $> call mysql.rds_set_configuration('binlog retention hours', 94);"
end
end
rescue Mysql2::Error => e
if e.message =~ /command denied to user/
log_warn_stderr("[WARNING]Cannot verify RDS retention period on current MySQL user account.\n" +
"To see retention period, please run this on your RDS:\n" +
" $> call mysql.rds_show_configuration;\n" +
"Please verify that the hours is not nil and is at least #{BINLOG_RETENTION_HOURS} hours\n" +
"To set binlog retention hours, you can run this on your RDS:\n" +
" $> call mysql.rds_set_configuration('binlog retention hours', #{BINLOG_RETENTION_HOURS});\n"
)
else
raise e
end
end
end
|