Class: Flydata::CompatibilityCheck
- Inherits:
-
Object
- Object
- Flydata::CompatibilityCheck
- Defined in:
- lib/flydata/compatibility_check.rb
Direct Known Subclasses
Defined Under Namespace
Classes: CompatibilityError
Instance Method Summary collapse
- #check ⇒ Object
-
#initialize(dp_hash, de_hash = nil, options = {}) ⇒ CompatibilityCheck
constructor
A new instance of CompatibilityCheck.
- #print_errors ⇒ Object
Constructor Details
#initialize(dp_hash, de_hash = nil, options = {}) ⇒ CompatibilityCheck
Returns a new instance of CompatibilityCheck.
7 8 9 10 |
# File 'lib/flydata/compatibility_check.rb', line 7 def initialize(dp_hash, de_hash=nil, ={}) @dp = dp_hash @errors=[] end |
Instance Method Details
#check ⇒ Object
12 13 14 15 16 17 18 19 20 21 |
# File 'lib/flydata/compatibility_check.rb', line 12 def check self.methods.grep(/^check_/).each do |m| begin send(m) rescue CompatibilityError => e @errors << e end end print_errors end |
#print_errors ⇒ Object
23 24 25 26 27 28 29 30 |
# File 'lib/flydata/compatibility_check.rb', line 23 def print_errors return if @errors.empty? puts "There may be some compatibility issues with this current server : " @errors.each do |error| puts " * #{error.}" end raise "Please correct these errors if you wish to run FlyData Agent" end |