Class: Flydata::Parser::Mysql::FlydataMysqlClient

Inherits:
Mysql2::Client
  • Object
show all
Defined in:
lib/flydata/parser/mysql/dump_parser.rb

Overview

Custom mysql client that gives access to the last query that was executed using the client which can be helpful when handling exceptions

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#last_query ⇒ Object

Returns the value of attribute last_query.



248
249
250
# File 'lib/flydata/parser/mysql/dump_parser.rb', line 248

def last_query
  @last_query
end

Instance Method Details

#query(sql, options = {}) ⇒ Object



250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
# File 'lib/flydata/parser/mysql/dump_parser.rb', line 250

def query(sql, options = {})
  @last_query = sql
  max_retry = options[:retry_count] || 0
  retry_interval = options[:retry_interval] || 3
  count = 0
  result = nil
  begin
    result = super(sql, options)
  rescue => e
    count += 1
    raise e if max_retry != -1 && count > max_retry
    yield(count, e) if block_given?
    sleep retry_interval
    retry
  end
  result
end