Class: Flydata::SyncFileManager

Inherits:
Object
  • Object
show all
Defined in:
lib/flydata/sync_file_manager.rb

Constant Summary collapse

DUMP_DIR =
ENV['FLYDATA_DUMP'] || File.join(FLYDATA_HOME, 'dump')
BACKUP_DIR =
ENV['FLYDATA_BACKUP'] || File.join(FLYDATA_HOME, 'backup')
TABLE_POSITIONS_DIR =
ENV['FLYDATA_TABLE_POSITIONS'] || File.join(FLYDATA_HOME, 'positions')

Instance Method Summary collapse

Constructor Details

#initialize(data_entry) ⇒ SyncFileManager

Returns a new instance of SyncFileManager.



8
9
10
11
# File 'lib/flydata/sync_file_manager.rb', line 8

def initialize(data_entry)
  @data_entry = data_entry
  @table_position_files = {} # File objects keyed by table name
end

Instance Method Details

#backup_dirObject



287
288
289
# File 'lib/flydata/sync_file_manager.rb', line 287

def backup_dir
  BACKUP_DIR
end

#backup_dump_dirObject



279
280
281
282
283
284
285
# File 'lib/flydata/sync_file_manager.rb', line 279

def backup_dump_dir
  backup_dir = BACKUP_DIR.dup
  FileUtils.mkdir_p(backup_dir) unless Dir.exists?(backup_dir)
  dest_dir = File.join(backup_dir, Time.now.strftime("%Y%m%d%H%M%S"))
  FileUtils.mkdir(dest_dir)
  FileUtils.mv(Dir.glob("#{dump_dir}/*"), dest_dir)
end

#binlog_pathObject



90
91
92
# File 'lib/flydata/sync_file_manager.rb', line 90

def binlog_path
  File.join(FLYDATA_HOME, @data_entry['name'] + ".binlog.pos")
end

#closeObject



13
14
15
16
# File 'lib/flydata/sync_file_manager.rb', line 13

def close
  @table_position_files.values.each {|f| f.close }
  @table_position_files = {}
end

#delete_dump_fileObject



275
276
277
# File 'lib/flydata/sync_file_manager.rb', line 275

def delete_dump_file
  FileUtils.rm(dump_file_path) if File.exists?(dump_file_path)
end

#delete_table_binlog_pos(table_name) ⇒ Object



242
243
244
245
246
247
248
249
# File 'lib/flydata/sync_file_manager.rb', line 242

def delete_table_binlog_pos(table_name)
  file = File.join(table_positions_dir_path, table_name + ".binlog.pos")
  if File.exists?(file)
    FileUtils.rm(file, :force => true)
  else
    puts "#{file} does not exist. Something is wrong. Did you delete the file manually when flydata was running?"
  end
end

#dump_file_pathObject



18
19
20
# File 'lib/flydata/sync_file_manager.rb', line 18

def dump_file_path
  File.join(dump_dir, @data_entry['name']) + ".dump"
end

#dump_pos_pathObject

dump pos file for resume



23
24
25
# File 'lib/flydata/sync_file_manager.rb', line 23

def dump_pos_path
  dump_file_path + ".pos"
end

#get_new_table_list(tables, file_type) ⇒ Object



55
56
57
58
59
60
61
62
# File 'lib/flydata/sync_file_manager.rb', line 55

def get_new_table_list(tables, file_type)
  table_positions_dir_path = ENV['FLYDATA_TABLE_POSITIONS'] || File.join(FLYDATA_HOME, 'positions')
  new_tables = []
  tables.each do |table|
    new_tables << table unless  File.exists?(File.join(table_positions_dir_path, "#{table}.#{file_type}"))
  end
  new_tables
end

#get_table_binlog_pos(table_name) ⇒ Object



205
206
207
208
209
# File 'lib/flydata/sync_file_manager.rb', line 205

def get_table_binlog_pos(table_name)
  file = File.join(table_positions_dir_path, table_name + ".binlog.pos")
  return nil unless File.exists?(file)
  File.open(file, 'r').readline
end

#increment_and_save_table_position(table_name) ⇒ Object

Read a sequence number from the table’s position file, increment the number and pass the number to a block. After executing the block, saves the value to the position file.



159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
# File 'lib/flydata/sync_file_manager.rb', line 159

def increment_and_save_table_position(table_name)
  file = File.join(table_positions_dir_path, table_name + ".pos")
  retry_count = 0
  begin
    @table_position_files[table_name] ||= File.open(file, "r+")
  rescue Errno::ENOENT
    raise if retry_count > 0 # Already retried.  Must be a differentfile causing the error
    # File not exist.  Create one with initial value of '0'
    File.open(file, "w") {|f| f.write('0') }
    retry_count += 1
    retry
  end
  f = @table_position_files[table_name]
  seq = f.read
  seq = seq.to_i + 1
  begin
    yield(seq)
  ensure
    # when an error happened in yield, the sequence number should remain
    # as is.  For the next call to read the value correctly, the position
    # must be rewound.
    f.rewind
  end
  f.truncate(0)
  f.write(seq)
  f.flush
  f.rewind
end

#increment_table_rev(table_name, base_rev) ⇒ Object



233
234
235
236
237
238
239
240
# File 'lib/flydata/sync_file_manager.rb', line 233

def increment_table_rev(table_name, base_rev)
  file = table_rev_file_path(table_name)
  new_rev = base_rev + 1
  File.open(file, "w") do |f|
    f.write(new_rev)
  end
  new_rev
end

#install_table_binlog_files(tables) ⇒ Object



260
261
262
263
264
265
266
267
268
269
270
271
272
273
# File 'lib/flydata/sync_file_manager.rb', line 260

def install_table_binlog_files(tables)
  FileUtils.mkdir_p(table_positions_dir_path) unless Dir.exists?(table_positions_dir_path)
  tables.each do |table_name|
    file_name = table_name + ".binlog.pos"
    src_file = File.join(dump_dir, file_name)
    if ! File.exists?(src_file)
      raise "#{src_file} does not exist. Error!!"
    end
    FileUtils.mv(src_file, table_positions_dir_path)
    # save the position at initial sync.  this is used for repair if
    # necessary.
    FileUtils.cp(File.join(table_positions_dir_path, file_name), File.join(table_positions_dir_path, file_name + ".init"))
  end
end

#load_binlog(file_path = binlog_path) ⇒ Object



83
84
85
86
87
88
# File 'lib/flydata/sync_file_manager.rb', line 83

def load_binlog(file_path = binlog_path)
  return nil unless File.exists?(file_path)
  f, pos = IO.read(file_path).strip.split("\t")
  return nil if f.nil? || f.empty? || pos.nil?
  { binfile: f, pos: pos.to_i }
end

#load_dump_posObject



33
34
35
36
37
38
39
40
41
42
# File 'lib/flydata/sync_file_manager.rb', line 33

def load_dump_pos
  path = dump_pos_path
  return {} unless File.exists?(path)
  items = File.open(path, 'r').readline.split("\t")
  raise "Invalid dump.pos file: #{path}" unless items.length >= 5 && items.length <= 7
  mysql_table = load_mysql_table_marshal_dump
  { status: items[0], table_name: items[1], last_pos: items[2].to_i,
    binlog_pos: {binfile: items[3], pos: items[4].to_i},
    state: items[5], substate: items[6], mysql_table: mysql_table}
end

#load_sync_infoObject



198
199
200
201
202
203
# File 'lib/flydata/sync_file_manager.rb', line 198

def load_sync_info
  return nil unless File.exists?(sync_info_file)
  items = File.open(sync_info_file, 'r').readline.split("\t")
  { initial_sync: (items[0] == 'true'),
    tables: items[1].split(" ") }
end

#mark_generated_tables(tables) ⇒ Object



44
45
46
47
48
49
50
51
52
53
# File 'lib/flydata/sync_file_manager.rb', line 44

def mark_generated_tables(tables)
  table_positions_dir_path = ENV['FLYDATA_TABLE_POSITIONS'] || File.join(FLYDATA_HOME, 'positions')
  #Create positions if dir does not exist
  unless File.directory?(table_positions_dir_path)
    FileUtils.mkdir_p(table_positions_dir_path)
  end
  tables.each do |tab|
    File.open(File.join(table_positions_dir_path, "#{tab}.generated_ddl"), 'w') {|f| f.write("1") }
  end
end

#mysql_table_marshal_dump_pathObject

MysqlTable marshal file



65
66
67
# File 'lib/flydata/sync_file_manager.rb', line 65

def mysql_table_marshal_dump_path
  dump_file_path + ".mysql_table"
end

#reset_table_position_files(tables) ⇒ Object

table files



124
125
126
127
128
129
# File 'lib/flydata/sync_file_manager.rb', line 124

def reset_table_position_files(tables)
  tables.each do |table_name|
    file = File.join(table_positions_dir_path, table_name + ".pos")
    File.open(file, "w") {|f| f.write('0') }
  end
end

#save_binlog(binlog_pos) ⇒ Object

master binlog.pos file



76
77
78
79
80
81
# File 'lib/flydata/sync_file_manager.rb', line 76

def save_binlog(binlog_pos)
  path = binlog_path
  File.open(path, 'w') do |f|
    f.write(binlog_content(binlog_pos))
  end
end

#save_dump_pos(status, table_name, last_pos, binlog_pos, state = nil, substate = nil) ⇒ Object



27
28
29
30
31
# File 'lib/flydata/sync_file_manager.rb', line 27

def save_dump_pos(status, table_name, last_pos, binlog_pos, state = nil, substate = nil)
  File.open(dump_pos_path, 'w') do |f|
    f.write(dump_pos_content(status, table_name, last_pos, binlog_pos, state, substate))
  end
end

#save_mysql_table_marshal_dump(mysql_table) ⇒ Object



69
70
71
72
73
# File 'lib/flydata/sync_file_manager.rb', line 69

def save_mysql_table_marshal_dump(mysql_table)
  File.open(mysql_table_marshal_dump_path, 'w') do |f|
    f.write Marshal.dump(mysql_table)
  end
end

#save_sent_binlog(binlog_pos) ⇒ Object

sent binlog.pos file



95
96
97
98
99
# File 'lib/flydata/sync_file_manager.rb', line 95

def save_sent_binlog(binlog_pos)
  File.open(sent_binlog_path, 'w') do |f|
    f.write(binlog_content(binlog_pos))
  end
end

#save_ssl_ca(ssl_ca_content, path = ssl_ca_path) ⇒ Object



117
118
119
120
121
# File 'lib/flydata/sync_file_manager.rb', line 117

def save_ssl_ca(ssl_ca_content, path = ssl_ca_path)
  File.open(path, 'w') do |f|
    f.write(ssl_ca_content)
  end
end

#save_sync_info(initial_sync, tables) ⇒ Object



192
193
194
195
196
# File 'lib/flydata/sync_file_manager.rb', line 192

def save_sync_info(initial_sync, tables)
  File.open(sync_info_file, "w") do |f|
    f.write([initial_sync, tables.join(" ")].join("\t"))
  end
end

#save_table_binlog_pos(tables, binlog_pos) ⇒ Object



251
252
253
254
255
256
257
258
# File 'lib/flydata/sync_file_manager.rb', line 251

def save_table_binlog_pos(tables, binlog_pos)
  tables.each do |table_name|
    file = File.join(dump_dir, table_name + ".binlog.pos")
    File.open(file, "w") do |f|
      f.write(binlog_content(binlog_pos))
    end
  end
end

#sent_binlog_path(master_binlog_path = binlog_path) ⇒ Object



101
102
103
104
105
106
# File 'lib/flydata/sync_file_manager.rb', line 101

def sent_binlog_path(master_binlog_path = binlog_path)
  unless master_binlog_path && master_binlog_path.end_with?('binlog.pos')
    raise ArgumentError.new("Invalid binlog path. binlog path needs to end with 'binlog.pos'")
  end
  "#{master_binlog_path[0..-5]}.sent.pos"
end

#ssl_ca_path(master_binlog_path = binlog_path) ⇒ Object

ssl_ca file path



109
110
111
112
113
114
115
# File 'lib/flydata/sync_file_manager.rb', line 109

def ssl_ca_path(master_binlog_path = binlog_path)
  unless master_binlog_path && master_binlog_path.end_with?('binlog.pos')
    raise ArgumentError.new("Invalid binlog path. binlog path needs to end with 'binlog.pos'")
  end
  # <data-entry-name>.ssl_ca.pem
  "#{master_binlog_path[0..-12]}.ssl_ca.pem"
end

#sync_info_fileObject



188
189
190
# File 'lib/flydata/sync_file_manager.rb', line 188

def sync_info_file
  File.join(dump_dir, "sync.info")
end

#table_binlog_pos_init_paths(*tables) ⇒ Object



150
151
152
153
# File 'lib/flydata/sync_file_manager.rb', line 150

def table_binlog_pos_init_paths(*tables)
  tables.empty? ? Dir.glob(File.join(table_positions_dir_path, '*.binlog.pos.init')) :
    tables.map{|table| File.join(table_positions_dir_path, table + '.binlog.pos.init')}
end

#table_binlog_pos_paths(*tables) ⇒ Object



145
146
147
148
# File 'lib/flydata/sync_file_manager.rb', line 145

def table_binlog_pos_paths(*tables)
  tables.empty? ? Dir.glob(File.join(table_positions_dir_path, '*.binlog.pos')) :
    tables.map{|table| File.join(table_positions_dir_path, table + '.binlog.pos')}
end

#table_ddl_file_paths(*tables) ⇒ Object



140
141
142
143
# File 'lib/flydata/sync_file_manager.rb', line 140

def table_ddl_file_paths(*tables)
  tables.empty? ? Dir.glob(File.join(table_positions_dir_path, '*.generated_ddl')) :
    tables.map{|table| File.join(table_positions_dir_path, table + '.generated_ddl')}
end

#table_position_file_paths(*tables) ⇒ Object



135
136
137
138
# File 'lib/flydata/sync_file_manager.rb', line 135

def table_position_file_paths(*tables)
  tables.empty? ? Dir.glob(File.join(table_positions_dir_path, '*.pos')) :
    tables.map{|table| File.join(table_positions_dir_path, table + '.pos')}
end

#table_positions_dir_pathObject



131
132
133
# File 'lib/flydata/sync_file_manager.rb', line 131

def table_positions_dir_path
  TABLE_POSITIONS_DIR
end

#table_rev(table_name) ⇒ Object



220
221
222
223
224
225
226
227
228
229
230
231
# File 'lib/flydata/sync_file_manager.rb', line 220

def table_rev(table_name)
  file = table_rev_file_path(table_name)
  return 1 unless File.exists?(file) #default revision is 1
  File.open(file, "r+") do |f|
    seq = f.read
    if seq.empty?
      return 1
    else
      return seq.to_i
    end
  end
end

#table_rev_file_path(table_name) ⇒ Object



211
212
213
# File 'lib/flydata/sync_file_manager.rb', line 211

def table_rev_file_path(table_name)
  File.join(table_positions_dir_path, table_name + ".rev")
end

#table_rev_file_paths(*tables) ⇒ Object



215
216
217
218
# File 'lib/flydata/sync_file_manager.rb', line 215

def table_rev_file_paths(*tables)
  tables.empty? ? Dir.glob(File.join(table_positions_dir_path, "*.rev")) :
    tables.map{|table| table_rev_file_path(table)}
end